import * as React from 'react';
import clsx from 'clsx';
import {slider} from './style.module.css';
import * as SelectControlBase from '@tesseract-design/web-base-selectcontrol';

type SliderOrientation = 'horizontal' | 'vertical';

interface RenderOptionsProps {
  options: (number | SelectControlBase.SelectOption)[],
  optionComponent?: React.ElementType,
  optgroupComponent?: React.ElementType,
  level?: number,
}

const RenderOptions: React.FC<RenderOptionsProps> = ({
  options,
  optionComponent: Option = 'option',
  optgroupComponent: Optgroup = 'optgroup',
  level = 0,
}: RenderOptionsProps) => (
  <>
    {
      options.map((o) => {
        if (typeof o === 'number') {
          return (
            <Option
              key={`${o}:${o}`}
              value={o}
            />
          )
        }

        if (typeof o.value !== 'undefined') {
          return (
            <Option
              key={`${o.label}:${o.value.toString()}`}
              value={o.value}
              label={o.label}
            >
              {o.label}
            </Option>
          );
        }

        if (typeof o.children !== 'undefined') {
          if (level === 0) {
            return (
              <Optgroup
                key={o.label}
                label={o.label}
              >
                <RenderOptions
                  options={o.children}
                  optionComponent={Option}
                  optgroupComponent={Optgroup}
                  level={level + 1}
                />
              </Optgroup>
            );
          }
          return (
            <React.Fragment
              key={o.label}
            >
              <Option
                disabled
              >
                {o.label}
              </Option>
              <RenderOptions
                options={o.children}
                optionComponent={Option}
                optgroupComponent={Optgroup}
                level={level + 1}
              />
            </React.Fragment>
          );
        }

        return null;
      })
    }
  </>
);

type SliderDerivedElement = HTMLInputElement;

export interface SliderProps extends Omit<React.HTMLProps<HTMLInputElement>, 'type'> {
  orient?: SliderOrientation;
  tickMarks?: (number | SelectControlBase.SelectOption)[];
  length?: React.CSSProperties['width'];
}

export const Slider = React.forwardRef<SliderDerivedElement, SliderProps>(({
  className,
  style,
  tickMarks = [],
  orient = 'horizontal',
  length,
  ...etcProps
}, forwardedRef) => {
  const [browser, setBrowser] = React.useState<string>();
  React.useEffect(() => {
    const isFirefox = typeof (window as unknown as Record<string, unknown>).InstallTrigger !== 'undefined';
    if (isFirefox) {
      setBrowser('firefox');
    } else {
      // TODO - detect other browsers
      setBrowser('chrome');
    }
  }, []);

  const defaultRef = React.useRef<HTMLInputElement>(null);
  const ref = forwardedRef ?? defaultRef;
  const tickMarkId = React.useId();

  React.useEffect(() => {
    if (!(typeof ref === 'object' && ref.current)) {
      return;
    }

    const {current: slider} = ref;
    const isFirefox = browser === 'firefox';
    const parent = slider?.parentElement as HTMLElement;
    const grandParent = parent?.parentElement as HTMLElement;
    if (isFirefox) {
      slider.setAttribute('orient', orient);
      slider.removeAttribute('data-orient');
      grandParent.style.width = '0px';
    }

    return () => {
      if (slider && isFirefox) {
        grandParent.style.width = 'auto';
        slider.dataset.orient = slider.getAttribute(orient) ?? undefined;
        slider.removeAttribute('orient');
      }
    };
  }, [ref, orient, browser]);

  React.useEffect(() => {
    if (!(typeof ref === 'object' && ref.current)) {
      return;
    }

    const {current: slider} = ref;
    const parent = slider?.parentElement as HTMLElement;
    const grandParent = parent?.parentElement as HTMLElement;

    const isNotFirefox = typeof browser === 'string' && browser !== 'firefox';
    const shouldEffectExecute = isNotFirefox && orient === 'vertical' && slider && parent && grandParent;
    if (shouldEffectExecute) {
      const trueHeight = parent.clientWidth;
      const trueWidth = parent.clientHeight;

      grandParent.dataset.height = grandParent.clientHeight.toString();
      grandParent.dataset.width = grandParent.clientWidth.toString();
      grandParent.style.height = trueHeight + 'px';
      grandParent.style.width = trueWidth + 'px';
    }

    return () => {
      if (shouldEffectExecute) {
        grandParent.style.height = grandParent.dataset.height + 'px';
        grandParent.style.width = grandParent.dataset.width + 'px';
        grandParent.removeAttribute('data-height');
        grandParent.removeAttribute('data-width');
      }
    };
  }, [ref, orient, browser]);

  const block = typeof length === 'string' && length.trim() === '100%';

  return (
    <>
      {
        tickMarks.length > 0
        && (
          <datalist
            id={tickMarkId}
          >
            <RenderOptions
              options={tickMarks}
            />
          </datalist>
        )
      }
      <div
        className={clsx(
          block && orient !== 'vertical' && 'w-full',
          !block && 'align-middle',
          className,
          orient !== 'vertical' && 'inline-block min-h-[1rem]',
          orient === 'vertical' && 'inline-block min-w-[1rem]',
          orient !== 'vertical' && !block && 'min-w-[16rem]',
          //orient === 'vertical' && 'inline-block min-h-[16rem] min-w-[1rem]',
        )}
        style={style}
      >
        <div
          style={{
            width: orient === 'vertical' ? (length ?? '16rem') : undefined,
            position: 'relative',
          }}
        >
          <input
            {...etcProps}
            ref={ref}
            type="range"
            data-orient={orient}
            className={clsx(
              slider,
              'w-full h-full bg-inherit block text-primary ring-secondary/50 rounded-full',
              'focus:text-secondary focus:outline-0 focus:ring-4',
              'active:text-tertiary active:ring-tertiary/50',
            )}
          />
        </div>
      </div>
    </>
  )
});

Slider.displayName = 'Slider';
